from maplibre.basemodel import Root
from maplibre.layer import Layer as L
from maplibre.sources import Source

sources = {
    "openmaptiles": Source(type="vector", url="https://api.maptiler.com/tiles/v3/tiles.json?key={key}"),
    "natural_earth_shaded_relief": Source(
        maxzoom=6,
        tileSize=256,
        tiles=["https://klokantech.github.io/naturalearthtiles/tiles/natural_earth_2_shaded_relief.raster/{z}/{x}/{y}.png"],
        type="raster",
    ),
}

layers = [
    L(type="background", id="background", paint={"background-color": "rgb(239,239,239)"}),
    L(id="natural_earth", type="raster", source="natural_earth_shaded_relief", maxzoom=6, paint={"raster-opacity": {"base": 1.5, "stops": [[0, 0.6], [6, 0.1]]}}),
    L(id="park", type="fill", source="openmaptiles", sourceLayer="park", paint={"fill-color": "#d8e8c8", "fill-opacity": 0.7, "fill-outline-color": "rgba(95, 208, 100, 1)"}),
    L(id="park_outline", type="line", source="openmaptiles", sourceLayer="park", paint={"line-dasharray": [1, 1.5], "line-color": "rgba(228, 241, 215, 1)"}),
    L(
        id="landuse_residential",
        type="fill",
        source="openmaptiles",
        sourceLayer="landuse",
        maxzoom=8,
        filter=["==", "class", "residential"],
        paint={"fill-color": {"base": 1, "stops": [[9, "hsla(0, 3%, 85%, 0.84)"], [12, "hsla(35, 57%, 88%, 0.49)"]]}},
    ),
    L(
        id="landcover_wood",
        type="fill",
        source="openmaptiles",
        sourceLayer="landcover",
        filter=["all", ["==", "class", "wood"]],
        paint={"fill-antialias": False, "fill-color": "hsla(98, 61%, 72%, 0.7)", "fill-opacity": 0.4},
    ),
    L(
        id="landcover_grass",
        type="fill",
        source="openmaptiles",
        sourceLayer="landcover",
        filter=["all", ["==", "class", "grass"]],
        paint={"fill-antialias": False, "fill-color": "rgba(176, 213, 154, 1)", "fill-opacity": 0.3},
    ),
    L(
        id="landcover_ice",
        type="fill",
        source="openmaptiles",
        sourceLayer="landcover",
        filter=["all", ["==", "class", "ice"]],
        paint={"fill-antialias": False, "fill-color": "rgba(224, 236, 236, 1)", "fill-opacity": 0.8},
    ),
    L(id="landuse_cemetery", type="fill", source="openmaptiles", sourceLayer="landuse", filter=["==", "class", "cemetery"], paint={"fill-color": "hsl(75, 37%, 81%)"}),
    L(id="landuse_hospital", type="fill", source="openmaptiles", sourceLayer="landuse", filter=["==", "class", "hospital"], paint={"fill-color": "#fde"}),
    L(id="landuse_school", type="fill", source="openmaptiles", sourceLayer="landuse", filter=["==", "class", "school"], paint={"fill-color": "rgb(236,238,204)"}),
    L(
        id="waterway_tunnel",
        type="line",
        source="openmaptiles",
        sourceLayer="waterway",
        filter=["all", ["==", "brunnel", "tunnel"]],
        paint={
            "line-color": "#a0c8f0",
            "line-dasharray": [3, 3],
            "line-gap-width": {"stops": [[12, 0], [20, 6]]},
            "line-opacity": 1,
            "line-width": {"base": 1.4, "stops": [[8, 1], [20, 2]]},
        },
    ),
    L(
        id="waterway_river",
        type="line",
        source="openmaptiles",
        sourceLayer="waterway",
        filter=["all", ["==", "class", "river"], ["!=", "brunnel", "tunnel"]],
        layout={"line-cap": "round"},
        paint={"line-color": "#a0c8f0", "line-width": {"base": 1.2, "stops": [[11, 0.5], [20, 6]]}},
    ),
    L(
        id="waterway_other",
        type="line",
        source="openmaptiles",
        sourceLayer="waterway",
        filter=["all", ["!=", "class", "river"], ["!=", "brunnel", "tunnel"]],
        layout={"line-cap": "round"},
        paint={"line-color": "#a0c8f0", "line-width": {"base": 1.3, "stops": [[13, 0.5], [20, 6]]}},
    ),
    L(id="water", type="fill", source="openmaptiles", sourceLayer="water", filter=["all", ["!=", "brunnel", "tunnel"]], paint={"fill-color": "rgb(158,189,255)"}),
    L(id="landcover_sand", type="fill", source="openmaptiles", sourceLayer="landcover", filter=["all", ["==", "class", "sand"]], paint={"fill-color": "rgba(247, 239, 195, 1)"}),
    L(
        id="aeroway_fill",
        type="fill",
        source="openmaptiles",
        sourceLayer="aeroway",
        minzoom=11,
        filter=["==", "$type", "Polygon"],
        paint={"fill-color": "rgba(229, 228, 224, 1)", "fill-opacity": 0.7},
    ),
    L(
        id="aeroway_runway",
        type="line",
        source="openmaptiles",
        sourceLayer="aeroway",
        minzoom=11,
        filter=["all", ["==", "$type", "LineString"], ["==", "class", "runway"]],
        paint={"line-color": "#f0ede9", "line-width": {"base": 1.2, "stops": [[11, 3], [20, 16]]}},
    ),
    L(
        id="aeroway_taxiway",
        type="line",
        source="openmaptiles",
        sourceLayer="aeroway",
        minzoom=11,
        filter=["all", ["==", "$type", "LineString"], ["==", "class", "taxiway"]],
        paint={"line-color": "#f0ede9", "line-width": {"base": 1.2, "stops": [[11, 0.5], [20, 6]]}},
    ),
    L(
        id="tunnel_motorway_link_casing",
        type="line",
        source="openmaptiles",
        sourceLayer="transportation",
        filter=["all", ["==", "class", "motorway"], ["==", "ramp", 1], ["==", "brunnel", "tunnel"]],
        layout={"line-join": "round"},
        paint={"line-color": "#e9ac77", "line-dasharray": [0.5, 0.25], "line-width": {"base": 1.2, "stops": [[12, 1], [13, 3], [14, 4], [20, 15]]}},
    ),
    L(
        id="tunnel_service_track_casing",
        type="line",
        source="openmaptiles",
        sourceLayer="transportation",
        filter=["all", ["==", "brunnel", "tunnel"], ["in", "class", "service", "track"]],
        layout={"line-join": "round"},
        paint={"line-color": "#cfcdca", "line-dasharray": [0.5, 0.25], "line-width": {"base": 1.2, "stops": [[15, 1], [16, 4], [20, 11]]}},
    ),
    L(
        id="tunnel_link_casing",
        type="line",
        source="openmaptiles",
        sourceLayer="transportation",
        filter=["all", ["==", "ramp", 1], ["==", "brunnel", "tunnel"]],
        layout={"line-join": "round"},
        paint={"line-color": "#e9ac77", "line-width": {"base": 1.2, "stops": [[12, 1], [13, 3], [14, 4], [20, 15]]}},
    ),
    L(
        id="tunnel_street_casing",
        type="line",
        source="openmaptiles",
        sourceLayer="transportation",
        filter=["all", ["==", "brunnel", "tunnel"], ["in", "class", "street", "street_limited"]],
        layout={"line-join": "round"},
        paint={"line-color": "#cfcdca", "line-opacity": {"stops": [[12, 0], [12.5, 1]]}, "line-width": {"base": 1.2, "stops": [[12, 0.5], [13, 1], [14, 4], [20, 15]]}},
    ),
    L(
        id="tunnel_secondary_tertiary_casing",
        type="line",
        source="openmaptiles",
        sourceLayer="transportation",
        filter=["all", ["==", "brunnel", "tunnel"], ["in", "class", "secondary", "tertiary"]],
        layout={"line-join": "round"},
        paint={"line-color": "#e9ac77", "line-width": {"base": 1.2, "stops": [[8, 1.5], [20, 17]]}},
    ),
    L(
        id="tunnel_trunk_primary_casing",
        type="line",
        source="openmaptiles",
        sourceLayer="transportation",
        filter=["all", ["==", "brunnel", "tunnel"], ["in", "class", "primary", "trunk"]],
        layout={"line-join": "round"},
        paint={"line-color": "#e9ac77", "line-width": {"base": 1.2, "stops": [[5, 0.4], [6, 0.7], [7, 1.75], [20, 22]]}},
    ),
    L(
        id="tunnel_motorway_casing",
        type="line",
        source="openmaptiles",
        sourceLayer="transportation",
        filter=["all", ["==", "class", "motorway"], ["!=", "ramp", 1], ["==", "brunnel", "tunnel"]],
        layout={"line-join": "round"},
        paint={"line-color": "#e9ac77", "line-dasharray": [0.5, 0.25], "line-width": {"base": 1.2, "stops": [[5, 0.4], [6, 0.7], [7, 1.75], [20, 22]]}},
    ),
    L(
        id="tunnel_path_pedestrian",
        type="line",
        source="openmaptiles",
        sourceLayer="transportation",
        filter=["all", ["==", "$type", "LineString"], ["==", "brunnel", "tunnel"], ["in", "class", "path", "pedestrian"]],
        paint={"line-color": "hsl(0, 0%, 100%)", "line-dasharray": [1, 0.75], "line-width": {"base": 1.2, "stops": [[14, 0.5], [20, 10]]}},
    ),
    L(
        id="tunnel_motorway_link",
        type="line",
        source="openmaptiles",
        sourceLayer="transportation",
        filter=["all", ["==", "class", "motorway"], ["==", "ramp", 1], ["==", "brunnel", "tunnel"]],
        layout={"line-join": "round"},
        paint={"line-color": "#fc8", "line-width": {"base": 1.2, "stops": [[12.5, 0], [13, 1.5], [14, 2.5], [20, 11.5]]}},
    ),
    L(
        id="tunnel_service_track",
        type="line",
        source="openmaptiles",
        sourceLayer="transportation",
        filter=["all", ["==", "brunnel", "tunnel"], ["in", "class", "service", "track"]],
        layout={"line-join": "round"},
        paint={"line-color": "#fff", "line-width": {"base": 1.2, "stops": [[15.5, 0], [16, 2], [20, 7.5]]}},
    ),
    L(
        id="tunnel_link",
        type="line",
        source="openmaptiles",
        sourceLayer="transportation",
        filter=["all", ["==", "ramp", 1], ["==", "brunnel", "tunnel"]],
        layout={"line-join": "round"},
        paint={"line-color": "#fff4c6", "line-width": {"base": 1.2, "stops": [[12.5, 0], [13, 1.5], [14, 2.5], [20, 11.5]]}},
    ),
    L(
        id="tunnel_minor",
        type="line",
        source="openmaptiles",
        sourceLayer="transportation",
        filter=["all", ["==", "brunnel", "tunnel"], ["in", "class", "minor"]],
        layout={"line-join": "round"},
        paint={"line-color": "#fff", "line-width": {"base": 1.2, "stops": [[13.5, 0], [14, 2.5], [20, 11.5]]}},
    ),
    L(
        id="tunnel_secondary_tertiary",
        type="line",
        source="openmaptiles",
        sourceLayer="transportation",
        filter=["all", ["==", "brunnel", "tunnel"], ["in", "class", "secondary", "tertiary"]],
        layout={"line-join": "round"},
        paint={"line-color": "#fff4c6", "line-width": {"base": 1.2, "stops": [[6.5, 0], [7, 0.5], [20, 10]]}},
    ),
    L(
        id="tunnel_trunk_primary",
        type="line",
        source="openmaptiles",
        sourceLayer="transportation",
        filter=["all", ["==", "brunnel", "tunnel"], ["in", "class", "primary", "trunk"]],
        layout={"line-join": "round"},
        paint={"line-color": "#fff4c6", "line-width": {"base": 1.2, "stops": [[5, 0], [7, 1], [20, 18]]}},
    ),
    L(
        id="tunnel_motorway",
        type="line",
        source="openmaptiles",
        sourceLayer="transportation",
        filter=["all", ["==", "class", "motorway"], ["!=", "ramp", 1], ["==", "brunnel", "tunnel"]],
        layout={"line-join": "round"},
        paint={"line-color": "#ffdaa6", "line-width": {"base": 1.2, "stops": [[5, 0], [7, 1], [20, 18]]}},
    ),
    L(
        id="tunnel_major_rail",
        type="line",
        source="openmaptiles",
        sourceLayer="transportation",
        filter=["all", ["==", "brunnel", "tunnel"], ["in", "class", "rail"]],
        paint={"line-color": "#bbb", "line-width": {"base": 1.4, "stops": [[14, 0.4], [15, 0.75], [20, 2]]}},
    ),
    L(
        id="tunnel_major_rail_hatching",
        type="line",
        source="openmaptiles",
        sourceLayer="transportation",
        filter=["all", ["==", "brunnel", "tunnel"], ["==", "class", "rail"]],
        paint={"line-color": "#bbb", "line-dasharray": [0.2, 8], "line-width": {"base": 1.4, "stops": [[14.5, 0], [15, 3], [20, 8]]}},
    ),
    L(
        id="tunnel_transit_rail",
        type="line",
        source="openmaptiles",
        sourceLayer="transportation",
        filter=["all", ["==", "brunnel", "tunnel"], ["in", "class", "transit"]],
        paint={"line-color": "#bbb", "line-width": {"base": 1.4, "stops": [[14, 0.4], [15, 0.75], [20, 2]]}},
    ),
    L(
        id="tunnel_transit_rail_hatching",
        type="line",
        source="openmaptiles",
        sourceLayer="transportation",
        filter=["all", ["==", "brunnel", "tunnel"], ["==", "class", "transit"]],
        paint={"line-color": "#bbb", "line-dasharray": [0.2, 8], "line-width": {"base": 1.4, "stops": [[14.5, 0], [15, 3], [20, 8]]}},
    ),
    L(
        id="road_area_pattern",
        type="fill",
        source="openmaptiles",
        sourceLayer="transportation",
        filter=["all", ["==", "$type", "Polygon"]],
        paint={"fill-pattern": "pedestrian_polygon"},
    ),
    L(
        id="road_motorway_link_casing",
        type="line",
        source="openmaptiles",
        sourceLayer="transportation",
        minzoom=12,
        filter=["all", ["!in", "brunnel", "bridge", "tunnel"], ["==", "class", "motorway"], ["==", "ramp", 1]],
        layout={"line-cap": "round", "line-join": "round"},
        paint={"line-color": "#e9ac77", "line-width": {"base": 1.2, "stops": [[12, 1], [13, 3], [14, 4], [20, 15]]}},
    ),
    L(
        id="road_service_track_casing",
        type="line",
        source="openmaptiles",
        sourceLayer="transportation",
        filter=["all", ["!in", "brunnel", "bridge", "tunnel"], ["in", "class", "service", "track"]],
        layout={"line-cap": "round", "line-join": "round"},
        paint={"line-color": "#cfcdca", "line-width": {"base": 1.2, "stops": [[15, 1], [16, 4], [20, 11]]}},
    ),
    L(
        id="road_link_casing",
        type="line",
        source="openmaptiles",
        sourceLayer="transportation",
        minzoom=13,
        filter=["all", ["!in", "brunnel", "bridge", "tunnel"], ["!in", "class", "pedestrian", "path", "track", "service", "motorway"], ["==", "ramp", 1]],
        layout={"line-cap": "round", "line-join": "round"},
        paint={"line-color": "#e9ac77", "line-width": {"base": 1.2, "stops": [[12, 1], [13, 3], [14, 4], [20, 15]]}},
    ),
    L(
        id="road_minor_casing",
        type="line",
        source="openmaptiles",
        sourceLayer="transportation",
        filter=["all", ["==", "$type", "LineString"], ["!in", "brunnel", "bridge", "tunnel"], ["in", "class", "minor"], ["!=", "ramp", 1]],
        layout={"line-cap": "round", "line-join": "round"},
        paint={"line-color": "#cfcdca", "line-opacity": {"stops": [[12, 0], [12.5, 1]]}, "line-width": {"base": 1.2, "stops": [[12, 0.5], [13, 1], [14, 4], [20, 20]]}},
    ),
    L(
        id="road_secondary_tertiary_casing",
        type="line",
        source="openmaptiles",
        sourceLayer="transportation",
        filter=["all", ["!in", "brunnel", "bridge", "tunnel"], ["in", "class", "secondary", "tertiary"], ["!=", "ramp", 1]],
        layout={"line-cap": "round", "line-join": "round"},
        paint={"line-color": "#e9ac77", "line-width": {"base": 1.2, "stops": [[8, 1.5], [20, 17]]}},
    ),
    L(
        id="road_trunk_primary_casing",
        type="line",
        source="openmaptiles",
        sourceLayer="transportation",
        filter=["all", ["!in", "brunnel", "bridge", "tunnel"], ["in", "class", "primary", "trunk"]],
        layout={"line-join": "round"},
        paint={"line-color": "#e9ac77", "line-width": {"base": 1.2, "stops": [[5, 0.4], [6, 0.7], [7, 1.75], [20, 22]]}},
    ),
    L(
        id="road_motorway_casing",
        type="line",
        source="openmaptiles",
        sourceLayer="transportation",
        minzoom=5,
        filter=["all", ["!in", "brunnel", "bridge", "tunnel"], ["==", "class", "motorway"], ["!=", "ramp", 1]],
        layout={"line-cap": "round", "line-join": "round"},
        paint={"line-color": "#e9ac77", "line-width": {"base": 1.2, "stops": [[5, 0.4], [6, 0.7], [7, 1.75], [20, 22]]}},
    ),
    L(
        id="road_path_pedestrian",
        type="line",
        source="openmaptiles",
        sourceLayer="transportation",
        minzoom=14,
        filter=["all", ["==", "$type", "LineString"], ["!in", "brunnel", "bridge", "tunnel"], ["in", "class", "path", "pedestrian"]],
        layout={"line-join": "round"},
        paint={"line-color": "hsl(0, 0%, 100%)", "line-dasharray": [1, 0.7], "line-width": {"base": 1.2, "stops": [[14, 1], [20, 10]]}},
    ),
    L(
        id="road_motorway_link",
        type="line",
        source="openmaptiles",
        sourceLayer="transportation",
        minzoom=12,
        filter=["all", ["!in", "brunnel", "bridge", "tunnel"], ["==", "class", "motorway"], ["==", "ramp", 1]],
        layout={"line-cap": "round", "line-join": "round"},
        paint={"line-color": "#fc8", "line-width": {"base": 1.2, "stops": [[12.5, 0], [13, 1.5], [14, 2.5], [20, 11.5]]}},
    ),
    L(
        id="road_service_track",
        type="line",
        source="openmaptiles",
        sourceLayer="transportation",
        filter=["all", ["!in", "brunnel", "bridge", "tunnel"], ["in", "class", "service", "track"]],
        layout={"line-cap": "round", "line-join": "round"},
        paint={"line-color": "#fff", "line-width": {"base": 1.2, "stops": [[15.5, 0], [16, 2], [20, 7.5]]}},
    ),
    L(
        id="road_link",
        type="line",
        source="openmaptiles",
        sourceLayer="transportation",
        minzoom=13,
        filter=["all", ["!in", "brunnel", "bridge", "tunnel"], ["==", "ramp", 1], ["!in", "class", "pedestrian", "path", "track", "service", "motorway"]],
        layout={"line-cap": "round", "line-join": "round"},
        paint={"line-color": "#fea", "line-width": {"base": 1.2, "stops": [[12.5, 0], [13, 1.5], [14, 2.5], [20, 11.5]]}},
    ),
    L(
        id="road_minor",
        type="line",
        source="openmaptiles",
        sourceLayer="transportation",
        filter=["all", ["==", "$type", "LineString"], ["!in", "brunnel", "bridge", "tunnel"], ["in", "class", "minor"]],
        layout={"line-cap": "round", "line-join": "round"},
        paint={"line-color": "#fff", "line-width": {"base": 1.2, "stops": [[13.5, 0], [14, 2.5], [20, 18]]}},
    ),
    L(
        id="road_secondary_tertiary",
        type="line",
        source="openmaptiles",
        sourceLayer="transportation",
        filter=["all", ["!in", "brunnel", "bridge", "tunnel"], ["in", "class", "secondary", "tertiary"]],
        layout={"line-cap": "round", "line-join": "round"},
        paint={"line-color": "#fea", "line-width": {"base": 1.2, "stops": [[6.5, 0], [8, 0.5], [20, 13]]}},
    ),
    L(
        id="road_trunk_primary",
        type="line",
        source="openmaptiles",
        sourceLayer="transportation",
        filter=["all", ["!in", "brunnel", "bridge", "tunnel"], ["in", "class", "primary", "trunk"]],
        layout={"line-join": "round"},
        paint={"line-color": "#fea", "line-width": {"base": 1.2, "stops": [[5, 0], [7, 1], [20, 18]]}},
    ),
    L(
        id="road_motorway",
        type="line",
        source="openmaptiles",
        sourceLayer="transportation",
        minzoom=5,
        filter=["all", ["!in", "brunnel", "bridge", "tunnel"], ["==", "class", "motorway"], ["!=", "ramp", 1]],
        layout={"line-cap": "round", "line-join": "round"},
        paint={"line-color": {"base": 1, "stops": [[5, "hsl(26, 87%, 62%)"], [6, "#fc8"]]}, "line-width": {"base": 1.2, "stops": [[5, 0], [7, 1], [20, 18]]}},
    ),
    L(
        id="road_major_rail",
        type="line",
        source="openmaptiles",
        sourceLayer="transportation",
        filter=["all", ["!in", "brunnel", "bridge", "tunnel"], ["==", "class", "rail"]],
        paint={"line-color": "#bbb", "line-width": {"base": 1.4, "stops": [[14, 0.4], [15, 0.75], [20, 2]]}},
    ),
    L(
        id="road_major_rail_hatching",
        type="line",
        source="openmaptiles",
        sourceLayer="transportation",
        filter=["all", ["!in", "brunnel", "bridge", "tunnel"], ["==", "class", "rail"]],
        paint={"line-color": "#bbb", "line-dasharray": [0.2, 8], "line-width": {"base": 1.4, "stops": [[14.5, 0], [15, 3], [20, 8]]}},
    ),
    L(
        id="road_transit_rail",
        type="line",
        source="openmaptiles",
        sourceLayer="transportation",
        filter=["all", ["!in", "brunnel", "bridge", "tunnel"], ["==", "class", "transit"]],
        paint={"line-color": "#bbb", "line-width": {"base": 1.4, "stops": [[14, 0.4], [15, 0.75], [20, 2]]}},
    ),
    L(
        id="road_transit_rail_hatching",
        type="line",
        source="openmaptiles",
        sourceLayer="transportation",
        filter=["all", ["!in", "brunnel", "bridge", "tunnel"], ["==", "class", "transit"]],
        paint={"line-color": "#bbb", "line-dasharray": [0.2, 8], "line-width": {"base": 1.4, "stops": [[14.5, 0], [15, 3], [20, 8]]}},
    ),
    L(
        id="road_one_way_arrow",
        type="symbol",
        source="openmaptiles",
        sourceLayer="transportation",
        minzoom=15,
        filter=["==", "oneway", 1],
        layout={"icon-image": "arrow", "symbol-placement": "line"},
    ),
    L(
        id="road_one_way_arrow_opposite",
        type="symbol",
        source="openmaptiles",
        sourceLayer="transportation",
        minzoom=15,
        filter=["==", "oneway", -1],
        layout={"icon-image": "arrow", "symbol-placement": "line", "icon-rotate": 180},
    ),
    L(
        id="bridge_motorway_link_casing",
        type="line",
        source="openmaptiles",
        sourceLayer="transportation",
        filter=["all", ["==", "class", "motorway"], ["==", "ramp", 1], ["==", "brunnel", "bridge"]],
        layout={"line-join": "round"},
        paint={"line-color": "#e9ac77", "line-width": {"base": 1.2, "stops": [[12, 1], [13, 3], [14, 4], [20, 15]]}},
    ),
    L(
        id="bridge_service_track_casing",
        type="line",
        source="openmaptiles",
        sourceLayer="transportation",
        filter=["all", ["==", "brunnel", "bridge"], ["in", "class", "service", "track"]],
        layout={"line-join": "round"},
        paint={"line-color": "#cfcdca", "line-width": {"base": 1.2, "stops": [[15, 1], [16, 4], [20, 11]]}},
    ),
    L(
        id="bridge_link_casing",
        type="line",
        source="openmaptiles",
        sourceLayer="transportation",
        filter=["all", ["==", "class", "link"], ["==", "brunnel", "bridge"]],
        layout={"line-join": "round"},
        paint={"line-color": "#e9ac77", "line-width": {"base": 1.2, "stops": [[12, 1], [13, 3], [14, 4], [20, 15]]}},
    ),
    L(
        id="bridge_street_casing",
        type="line",
        source="openmaptiles",
        sourceLayer="transportation",
        filter=["all", ["==", "brunnel", "bridge"], ["in", "class", "street", "street_limited"]],
        layout={"line-join": "round"},
        paint={"line-color": "hsl(36, 6%, 74%)", "line-opacity": {"stops": [[12, 0], [12.5, 1]]}, "line-width": {"base": 1.2, "stops": [[12, 0.5], [13, 1], [14, 4], [20, 25]]}},
    ),
    L(
        id="bridge_path_pedestrian_casing",
        type="line",
        source="openmaptiles",
        sourceLayer="transportation",
        filter=["all", ["==", "$type", "LineString"], ["==", "brunnel", "bridge"], ["in", "class", "path", "pedestrian"]],
        paint={"line-color": "hsl(35, 6%, 80%)", "line-dasharray": [1, 0], "line-width": {"base": 1.2, "stops": [[14, 1.5], [20, 18]]}},
    ),
    L(
        id="bridge_secondary_tertiary_casing",
        type="line",
        source="openmaptiles",
        sourceLayer="transportation",
        filter=["all", ["==", "brunnel", "bridge"], ["in", "class", "secondary", "tertiary"]],
        layout={"line-join": "round"},
        paint={"line-color": "#e9ac77", "line-width": {"base": 1.2, "stops": [[8, 1.5], [20, 17]]}},
    ),
    L(
        id="bridge_trunk_primary_casing",
        type="line",
        source="openmaptiles",
        sourceLayer="transportation",
        filter=["all", ["==", "brunnel", "bridge"], ["in", "class", "primary", "trunk"]],
        layout={"line-join": "round"},
        paint={"line-color": "#e9ac77", "line-width": {"base": 1.2, "stops": [[5, 0.4], [6, 0.7], [7, 1.75], [20, 22]]}},
    ),
    L(
        id="bridge_motorway_casing",
        type="line",
        source="openmaptiles",
        sourceLayer="transportation",
        filter=["all", ["==", "class", "motorway"], ["!=", "ramp", 1], ["==", "brunnel", "bridge"]],
        layout={"line-join": "round"},
        paint={"line-color": "#e9ac77", "line-width": {"base": 1.2, "stops": [[5, 0.4], [6, 0.7], [7, 1.75], [20, 22]]}},
    ),
    L(
        id="bridge_path_pedestrian",
        type="line",
        source="openmaptiles",
        sourceLayer="transportation",
        filter=["all", ["==", "$type", "LineString"], ["==", "brunnel", "bridge"], ["in", "class", "path", "pedestrian"]],
        paint={"line-color": "hsl(0, 0%, 100%)", "line-dasharray": [1, 0.3], "line-width": {"base": 1.2, "stops": [[14, 0.5], [20, 10]]}},
    ),
    L(
        id="bridge_motorway_link",
        type="line",
        source="openmaptiles",
        sourceLayer="transportation",
        filter=["all", ["==", "class", "motorway"], ["==", "ramp", 1], ["==", "brunnel", "bridge"]],
        layout={"line-join": "round"},
        paint={"line-color": "#fc8", "line-width": {"base": 1.2, "stops": [[12.5, 0], [13, 1.5], [14, 2.5], [20, 11.5]]}},
    ),
    L(
        id="bridge_service_track",
        type="line",
        source="openmaptiles",
        sourceLayer="transportation",
        filter=["all", ["==", "brunnel", "bridge"], ["in", "class", "service", "track"]],
        layout={"line-join": "round"},
        paint={"line-color": "#fff", "line-width": {"base": 1.2, "stops": [[15.5, 0], [16, 2], [20, 7.5]]}},
    ),
    L(
        id="bridge_link",
        type="line",
        source="openmaptiles",
        sourceLayer="transportation",
        filter=["all", ["==", "class", "link"], ["==", "brunnel", "bridge"]],
        layout={"line-join": "round"},
        paint={"line-color": "#fea", "line-width": {"base": 1.2, "stops": [[12.5, 0], [13, 1.5], [14, 2.5], [20, 11.5]]}},
    ),
    L(
        id="bridge_street",
        type="line",
        source="openmaptiles",
        sourceLayer="transportation",
        filter=["all", ["==", "brunnel", "bridge"], ["in", "class", "minor"]],
        layout={"line-join": "round"},
        paint={"line-color": "#fff", "line-width": {"base": 1.2, "stops": [[13.5, 0], [14, 2.5], [20, 18]]}},
    ),
    L(
        id="bridge_secondary_tertiary",
        type="line",
        source="openmaptiles",
        sourceLayer="transportation",
        filter=["all", ["==", "brunnel", "bridge"], ["in", "class", "secondary", "tertiary"]],
        layout={"line-join": "round"},
        paint={"line-color": "#fea", "line-width": {"base": 1.2, "stops": [[6.5, 0], [7, 0.5], [20, 10]]}},
    ),
    L(
        id="bridge_trunk_primary",
        type="line",
        source="openmaptiles",
        sourceLayer="transportation",
        filter=["all", ["==", "brunnel", "bridge"], ["in", "class", "primary", "trunk"]],
        layout={"line-join": "round"},
        paint={"line-color": "#fea", "line-width": {"base": 1.2, "stops": [[5, 0], [7, 1], [20, 18]]}},
    ),
    L(
        id="bridge_motorway",
        type="line",
        source="openmaptiles",
        sourceLayer="transportation",
        filter=["all", ["==", "class", "motorway"], ["!=", "ramp", 1], ["==", "brunnel", "bridge"]],
        layout={"line-join": "round"},
        paint={"line-color": "#fc8", "line-width": {"base": 1.2, "stops": [[5, 0], [7, 1], [20, 18]]}},
    ),
    L(
        id="bridge_major_rail",
        type="line",
        source="openmaptiles",
        sourceLayer="transportation",
        filter=["all", ["==", "class", "rail"], ["==", "brunnel", "bridge"]],
        paint={"line-color": "#bbb", "line-width": {"base": 1.4, "stops": [[14, 0.4], [15, 0.75], [20, 2]]}},
    ),
    L(
        id="bridge_major_rail_hatching",
        type="line",
        source="openmaptiles",
        sourceLayer="transportation",
        filter=["all", ["==", "class", "rail"], ["==", "brunnel", "bridge"]],
        paint={"line-color": "#bbb", "line-dasharray": [0.2, 8], "line-width": {"base": 1.4, "stops": [[14.5, 0], [15, 3], [20, 8]]}},
    ),
    L(
        id="bridge_transit_rail",
        type="line",
        source="openmaptiles",
        sourceLayer="transportation",
        filter=["all", ["==", "class", "transit"], ["==", "brunnel", "bridge"]],
        paint={"line-color": "#bbb", "line-width": {"base": 1.4, "stops": [[14, 0.4], [15, 0.75], [20, 2]]}},
    ),
    L(
        id="bridge_transit_rail_hatching",
        type="line",
        source="openmaptiles",
        sourceLayer="transportation",
        filter=["all", ["==", "class", "transit"], ["==", "brunnel", "bridge"]],
        paint={"line-color": "#bbb", "line-dasharray": [0.2, 8], "line-width": {"base": 1.4, "stops": [[14.5, 0], [15, 3], [20, 8]]}},
    ),
    L(
        id="building",
        type="fill",
        source="openmaptiles",
        sourceLayer="building",
        minzoom=13,
        maxzoom=14,
        paint={"fill-color": "hsl(35, 8%, 85%)", "fill-outline-color": {"base": 1, "stops": [[13, "hsla(35, 6%, 79%, 0.32)"], [14, "hsl(35, 6%, 79%)"]]}},
    ),
    L(
        id="building-3d",
        type="fill-extrusion",
        source="openmaptiles",
        sourceLayer="building",
        minzoom=14,
        paint={
            "fill-extrusion-color": "hsl(35, 8%, 85%)",
            "fill-extrusion-height": {"property": "render_height", "type": "identity"},
            "fill-extrusion-base": {"property": "render_min_height", "type": "identity"},
            "fill-extrusion-opacity": 0.8,
        },
    ),
    L(
        id="boundary_3",
        type="line",
        source="openmaptiles",
        sourceLayer="boundary",
        minzoom=8,
        filter=["all", ["in", "admin_level", 3, 4]],
        layout={"line-join": "round"},
        paint={"line-color": "#9e9cab", "line-dasharray": [5, 1], "line-width": {"base": 1, "stops": [[4, 0.4], [5, 1], [12, 1.8]]}},
    ),
    L(
        id="boundary_2_z0-4",
        type="line",
        source="openmaptiles",
        sourceLayer="boundary",
        maxzoom=5,
        filter=["all", ["==", "admin_level", 2], ["!has", "claimed_by"]],
        layout={"line-cap": "round", "line-join": "round"},
        paint={"line-color": "hsl(248, 1%, 41%)", "line-opacity": {"base": 1, "stops": [[0, 0.4], [4, 1]]}, "line-width": {"base": 1, "stops": [[3, 1], [5, 1.2], [12, 3]]}},
    ),
    L(
        id="boundary_2_z5-",
        type="line",
        source="openmaptiles",
        sourceLayer="boundary",
        minzoom=5,
        filter=["all", ["==", "admin_level", 2]],
        layout={"line-cap": "round", "line-join": "round"},
        paint={"line-color": "hsl(248, 1%, 41%)", "line-opacity": {"base": 1, "stops": [[0, 0.4], [4, 1]]}, "line-width": {"base": 1, "stops": [[3, 1], [5, 1.2], [12, 3]]}},
    ),
    L(
        id="water_name_line",
        type="symbol",
        source="openmaptiles",
        sourceLayer="waterway",
        filter=["all", ["==", "$type", "LineString"]],
        layout={"text-field": "{name}", "text-font": ["Roboto Regular"], "text-max-width": 5, "text-size": 12, "symbol-placement": "line"},
        paint={"text-color": "#5d60be", "text-halo-color": "rgba(255,255,255,0.7)", "text-halo-width": 1},
    ),
    L(
        id="water_name_point",
        type="symbol",
        source="openmaptiles",
        sourceLayer="water_name",
        filter=["==", "$type", "Point"],
        layout={"text-field": "{name}", "text-font": ["Roboto Regular"], "text-max-width": 5, "text-size": 12},
        paint={"text-color": "#5d60be", "text-halo-color": "rgba(255,255,255,0.7)", "text-halo-width": 1},
    ),
    L(
        id="poi_z16",
        type="symbol",
        source="openmaptiles",
        sourceLayer="poi",
        minzoom=16,
        filter=["all", ["==", "$type", "Point"], [">=", "rank", 20]],
        layout={
            "icon-image": "{class}_11",
            "text-anchor": "top",
            "text-field": "{name}",
            "text-font": ["Roboto Condensed Italic"],
            "text-max-width": 9,
            "text-offset": [0, 0.6],
            "text-size": 12,
        },
        paint={"text-color": "#666", "text-halo-blur": 0.5, "text-halo-color": "#ffffff", "text-halo-width": 1},
    ),
    L(
        id="poi_z15",
        type="symbol",
        source="openmaptiles",
        sourceLayer="poi",
        minzoom=15,
        filter=["all", ["==", "$type", "Point"], [">=", "rank", 7], ["<", "rank", 20]],
        layout={
            "icon-image": "{class}_11",
            "text-anchor": "top",
            "text-field": "{name}",
            "text-font": ["Roboto Condensed Italic"],
            "text-max-width": 9,
            "text-offset": [0, 0.6],
            "text-size": 12,
        },
        paint={"text-color": "#666", "text-halo-blur": 0.5, "text-halo-color": "#ffffff", "text-halo-width": 1},
    ),
    L(
        id="poi_z14",
        type="symbol",
        source="openmaptiles",
        sourceLayer="poi",
        minzoom=14,
        filter=["all", ["==", "$type", "Point"], [">=", "rank", 1], ["<", "rank", 7]],
        layout={
            "icon-image": "{class}_11",
            "text-anchor": "top",
            "text-field": "{name}",
            "text-font": ["Roboto Condensed Italic"],
            "text-max-width": 9,
            "text-offset": [0, 0.6],
            "text-size": 12,
        },
        paint={"text-color": "#666", "text-halo-blur": 0.5, "text-halo-color": "#ffffff", "text-halo-width": 1},
    ),
    L(
        id="poi_transit",
        type="symbol",
        source="openmaptiles",
        sourceLayer="poi",
        filter=["all", ["in", "class", "bus", "rail", "airport"]],
        layout={
            "icon-image": "{class}_11",
            "text-anchor": "left",
            "text-field": "{name_en}",
            "text-font": ["Roboto Condensed Italic"],
            "text-max-width": 9,
            "text-offset": [0.9, 0],
            "text-size": 12,
        },
        paint={"text-color": "#4898ff", "text-halo-blur": 0.5, "text-halo-color": "#ffffff", "text-halo-width": 1},
    ),
    L(
        id="road_label",
        type="symbol",
        source="openmaptiles",
        sourceLayer="transportation_name",
        filter=["all"],
        layout={
            "symbol-placement": "line",
            "text-anchor": "center",
            "text-field": "{name}",
            "text-font": ["Roboto Regular"],
            "text-offset": [0, 0.15],
            "text-size": {"base": 1, "stops": [[13, 12], [14, 13]]},
        },
        paint={"text-color": "#765", "text-halo-blur": 0.5, "text-halo-width": 1},
    ),
    L(
        id="road_shield",
        type="symbol",
        source="openmaptiles",
        sourceLayer="transportation_name",
        minzoom=7,
        filter=["all", ["<=", "ref_length", 6]],
        layout={
            "icon-image": "default_{ref_length}",
            "icon-rotation-alignment": "viewport",
            "symbol-placement": {"base": 1, "stops": [[10, "point"], [11, "line"]]},
            "symbol-spacing": 500,
            "text-field": "{ref}",
            "text-font": ["Roboto Regular"],
            "text-offset": [0, 0.1],
            "text-rotation-alignment": "viewport",
            "text-size": 10,
            "icon-size": 0.8,
        },
    ),
    L(
        id="place_other",
        type="symbol",
        source="openmaptiles",
        sourceLayer="place",
        filter=["all", ["in", "class", "hamlet", "island", "islet", "neighbourhood", "suburb", "quarter"]],
        layout={
            "text-field": "{name_en}",
            "text-font": ["Roboto Condensed Italic"],
            "text-letter-spacing": 0.1,
            "text-max-width": 9,
            "text-size": {"base": 1.2, "stops": [[12, 10], [15, 14]]},
            "text-transform": "uppercase",
        },
        paint={"text-color": "#633", "text-halo-color": "rgba(255,255,255,0.8)", "text-halo-width": 1.2},
    ),
    L(
        id="place_village",
        type="symbol",
        source="openmaptiles",
        sourceLayer="place",
        filter=["all", ["==", "class", "village"]],
        layout={"text-field": "{name_en}", "text-font": ["Roboto Regular"], "text-max-width": 8, "text-size": {"base": 1.2, "stops": [[10, 12], [15, 22]]}},
        paint={"text-color": "#333", "text-halo-color": "rgba(255,255,255,0.8)", "text-halo-width": 1.2},
    ),
    L(
        id="place_town",
        type="symbol",
        source="openmaptiles",
        sourceLayer="place",
        filter=["all", ["==", "class", "town"]],
        layout={
            "icon-image": {"base": 1, "stops": [[0, "dot_9"], [8, ""]]},
            "text-anchor": "bottom",
            "text-field": "{name_en}",
            "text-font": ["Roboto Regular"],
            "text-max-width": 8,
            "text-offset": [0, 0],
            "text-size": {"base": 1.2, "stops": [[7, 12], [11, 16]]},
        },
        paint={"text-color": "#333", "text-halo-color": "rgba(255,255,255,0.8)", "text-halo-width": 1.2},
    ),
    L(
        id="place_city",
        type="symbol",
        source="openmaptiles",
        sourceLayer="place",
        minzoom=5,
        filter=["all", ["==", "class", "city"]],
        layout={
            "icon-image": {"base": 1, "stops": [[0, "dot_9"], [8, ""]]},
            "text-anchor": "bottom",
            "text-field": "{name_en}",
            "text-font": ["Roboto Medium"],
            "text-max-width": 8,
            "text-offset": [0, 0],
            "text-size": {"base": 1.2, "stops": [[7, 14], [11, 24]]},
            "icon-allow-overlap": True,
            "icon-optional": False,
        },
        paint={"text-color": "#333", "text-halo-color": "rgba(255,255,255,0.8)", "text-halo-width": 1.2},
    ),
    L(
        id="state",
        type="symbol",
        source="openmaptiles",
        sourceLayer="place",
        maxzoom=6,
        filter=["all", ["==", "class", "state"]],
        layout={"text-field": "{name_en}", "text-font": ["Roboto Condensed Italic"], "text-size": {"stops": [[4, 11], [6, 15]]}, "text-transform": "uppercase"},
        paint={"text-color": "#633", "text-halo-color": "rgba(255,255,255,0.7)", "text-halo-width": 1},
    ),
    L(
        id="country_3",
        type="symbol",
        source="openmaptiles",
        sourceLayer="place",
        filter=["all", [">=", "rank", 3], ["==", "class", "country"]],
        layout={"text-field": "{name_en}", "text-font": ["Roboto Condensed Italic"], "text-max-width": 6.25, "text-size": {"stops": [[3, 11], [7, 17]]}, "text-transform": "none"},
        paint={"text-color": "#334", "text-halo-blur": 1, "text-halo-color": "rgba(255,255,255,0.8)", "text-halo-width": 1},
    ),
    L(
        id="country_2",
        type="symbol",
        source="openmaptiles",
        sourceLayer="place",
        filter=["all", ["==", "rank", 2], ["==", "class", "country"]],
        layout={"text-field": "{name_en}", "text-font": ["Roboto Condensed Italic"], "text-max-width": 6.25, "text-size": {"stops": [[2, 11], [5, 17]]}, "text-transform": "none"},
        paint={"text-color": "#334", "text-halo-blur": 1, "text-halo-color": "rgba(255,255,255,0.8)", "text-halo-width": 1},
    ),
    L(
        id="country_1",
        type="symbol",
        source="openmaptiles",
        sourceLayer="place",
        filter=["all", ["==", "rank", 1], ["==", "class", "country"]],
        layout={"text-field": "{name_en}", "text-font": ["Roboto Condensed Italic"], "text-max-width": 6.25, "text-size": {"stops": [[1, 11], [4, 17]]}, "text-transform": "none"},
        paint={"text-color": "#334", "text-halo-blur": 1, "text-halo-color": "rgba(255,255,255,0.8)", "text-halo-width": 1},
    ),
    L(
        id="continent",
        type="symbol",
        source="openmaptiles",
        sourceLayer="place",
        maxzoom=1,
        filter=["all", ["==", "class", "continent"]],
        layout={"text-field": "{name_en}", "text-font": ["Roboto Condensed Italic"], "text-size": 13, "text-transform": "uppercase", "text-justify": "center"},
        paint={"text-color": "#633", "text-halo-color": "rgba(255,255,255,0.7)", "text-halo-width": 1},
    ),
]

map = Root(
    version=8,
    name="OSM Liberty",
    metadata={"maputnik:license": "https://github.com/maputnik/osm-liberty/blob/gh-pages/LICENSE.md", "maputnik:renderer": "mbgljs", "openmaptiles:version": "3.x"},
    sprite="https://maputnik.github.io/osm-liberty/sprites/osm-liberty",
    glyphs="https://api.maptiler.com/fonts/{fontstack}/{range}.pbf?key={key}",
    sources=sources,
    layers=layers,
)



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['djangostreetmap',
 'djangostreetmap.management.commands',
 'djangostreetmap.maplibre',
 'djangostreetmap.migrations',
 'maplibre']

package_data = \
{'': ['*'], 'djangostreetmap': ['templates/*']}

install_requires = \
['osmflex>=0.2.0,<0.3.0', 'pydantic>=1.10.0,<2.0.0']

setup_kwargs = {
    'name': 'djangostreetmap',
    'version': '0.2.7',
    'description': 'Deliver OpenstreetMap data in GeoJSON and MVT tile formats',
    'long_description': '[![codecov](https://codecov.io/gh/joshbrooks/djangostreetmap/branch/main/graph/badge.svg?token=MXcJUkbOMf)](https://codecov.io/gh/joshbrooks/djangostreetmap)\n\n# DjangoStreetMap\n\nDjangSstreetMap is a Django application to load OSM data into a postgis database and deliver OSM data as\nMVT tiles.\n\n## Openstreetmap Vector Tiles\n\n> "A vector tile is a lightweight data format for storing geospatial vector data"\n\nFor an introduction to MVT (Mapbox Vector Tiles) see the [mapbox docs](https://docs.mapbox.com/help/glossary/vector-tiles/)\nFor an introduction to OSM see [openstreetmap.org](https://www.openstreetmap.org/)\n\n## Purpose of this project\n\nThis is a Django application to\n\n1. Import OSM data as Django models\n2. Expose Django models as MVT (Mapbox Vector Tile) geographic format data\n\nTile generation is much faster when geometry is in srid=3857 (or maybe with an index in that SRID?)\n\n## Prerequisites\n\nYou need the `gdal` libraries installed\n\nOn Ubuntu:\n```\nsudo apt install binutils libproj-dev gdal-bin\n```\n\nOtherwise refer to the Django docs "Installing geospatial libraries"\n## Adding to a Project\n\nIf necessary install psycopg2 in your env\n\nExtend installed_apps with the following apps:\n\n`pip install osmflex`\n\n```python\n[\n    "django.contrib.gis",\n    "djangostreetmap",\n    "osmflex",\n]\n```\n\n## (Recommended) Set your cache\n\nYou likely want to set a fast cache for your tiles like Memcached. If this is not found\nthe default cache will be used; this can be a bit slower and is very much non persistent\nThis assumes you\'re running memcached (Linux: `apt install memcached`) and installed memcached(`pip install python-memcached`)\n\n```python\nCACHES = {\n    \'default\': {\n        \'BACKEND\': \'django.core.cache.backends.memcached.MemcachedCache\',\n        \'LOCATION\': \'127.0.0.1:11211\',\n    }\n}\n```\n\n## Running faster in testing\n\nRun `poetry install`\n\nTo run pytest, you need to have an appropriate postgis database\nIf you use docker one option is to run the following:\n\n```bash\ndocker run \\\n    --rm \\\n    -p 49155:5432 \\\n    --name=djangostreetmap \\\n    -e POSTGRES_PASSWORD=post1234 \\\n    postgis/postgis:14-3.2 \\\n    -c fsync=off \\\n    -c shared_buffers=4096MB\n```\nRun `poetry run pytest`\n\n\nRunserver is "ok" but this recipe will give faster performance for demonstration purposes\n\n\n\n```bash\npip install gunicorn\ngunicorn -w 8 djangostreetmap.wsgi:application\n```\n\n## Building\n\npoetry version patch\npoetry build\npoetry publish\n\n## Writing Views\n\nTo set up a new View, create a subclass of TileLayerView with some `MvtQuery` instances as layers:\n\n```python\nclass RoadLayerView(TileLayerView):\n    layers = [\n        MvtQuery(table=OsmHighway._meta.db_table, attributes=["name"], filters=[f"\\"highway\\"=\'{road_class}\'"], layer=road_class)\n        for road_class in ("primary", "secondary", "tertiary")\n    ]\n```\n\nAppend the URL to your urls.py as follows. Note the zoom, x and y are mandatory.\n\n```python\n    path("highways/<int:zoom>/<int:x>/<int:y>.pbf", RoadLayerView.as_view()),\n```\n\n## Running in Development\n\n### Set up postgis\n\n```bash\ndocker run --name=osm \\\n    -e POSTGRES_DB=postgres \\\n    -e POSTGRES_USER=postgres \\\n    -e POSTGRES_PASSWORD=post1234 \\\n    -p 49155:5432 \\\n    postgis/postgis:12-3.1\n```\n\nFind your port: if you do not use `49155` as above:\n\n```sh\n(env) josh@m4800:~/github/joshbrooks/djangostreetmap$ docker ps\nCONTAINER ID   IMAGE                            COMMAND                  CREATED          STATUS             PORTS                                         NAMES\nc619232fe38a   postgis/postgis:12-3.1           "docker-entrypoint.s…"   33 seconds ago   Up 32 seconds      0.0.0.0:49155->5432/tcp, :::49155->5432/tcp   osm\n...\n```\n\nOSM is on port 49155\n\nTo apply this to your project:\n\n```python\n  DATABASES = {\n    "default": {\n        "ENGINE": "django.contrib.gis.db.backends.postgis",\n        "USER": "postgres",\n        "PASSWORD": "post1233",\n        "HOST": "localhost",\n        "PORT": "49154",\n        "NAME": "postgres",\n    }\n}\n```\n\n### Fetch your data\n\nwget https://download.geofabrik.de/australia-oceania/papua-new-guinea-latest.osm.pbf\n\nor\n\nwget https://download.geofabrik.de/asia/east-timor-latest.osm.pbf\n\n### Installing osm2psql\n\nTo run the management command below you\'ll need an `osm2pgsql` version of around 1.3 or greater. This is not available in the ubuntu package manager (yet)...\n\n### Import Data\n\nThe "osmflex" app has two management commands to run which will populate osmflex models\n\n```sh\n./manage.py run_osm2pgsql /media/josh/blackgate/osm/asia/east-timor-latest.osm.pbf\n```\n\n```sh\n./manage.py import_from_pgosmflex\n```\n\n### Exploring data\n\nSee the Django admin for osmflex:\n\nhttp://localhost:8000/admin/osmflex\n\npsql --host localhost --username postgres --port 49159\n\n### Qgis\n\n- Add a new Postgres Connection with the following settings:\n\nName: DjangoStreetMap\nHost: localhost\nPort: 49155\nDatabase: postgres\n\nAuthentication: Basic\npostgres / post1233\n\n## Development\n\nCode is blacked, flaked, isorted and mypy\'d.\n\n`pip install pre-commit`\n',
    'author': 'Joshua Brooks',
    'author_email': 'josh.vdbroek@gmail.com',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'https://github.com/joshbrooks/djangostreetmap',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)



from dataclasses import asdict
from http import HTTPStatus
from typing import List, Tuple

from django.contrib.gis.geos import Point
from django.db import connection
from django.db.models import Q
from django.test import TestCase
from django.urls import reverse
from osmflex.models import RoadLine
from psycopg2 import sql

from djangostreetmap.tilegenerator import MvtQuery, Tile
from tests.models import BasicPoint

# This reference is from /14/14891/8624, around 'Five Mile', Port Moresby
port_moresby = Tile(zoom=14, x=14891, y=8624)

# Specify different zoom levels for road types
road_osm_types: List[Tuple[str, int]] = [
    ("trunk", 2),
    ("steps", 12),
    ("road", 12),
    ("footway", 12),
    ("secondary", 7),
    ("tertiary", 9),
    ("secondary_link", 7),
    ("tertiary_link", 9),
    ("living_street", 12),
    ("pedestrian", 12),
    ("primary", 5),
    ("residential", 13),
    ("primary_link", 5),
    ("track", 12),
    ("motorway_link", 12),
    ("motorway", 5),
    ("service", 12),
    ("unclassified", 12),
    ("path", 12),
]


class RoadMvtTestCase(TestCase):
    def test_roads_model(self):

        road_types = [road_type for road_type, min_zoom in road_osm_types if port_moresby.zoom > min_zoom]

        # Build an array for the query
        type_filter = sql.Identifier("osm_type") + sql.SQL(" = ANY ") + sql.SQL("(ARRAY[")
        type_filter += sql.SQL(",").join([sql.Literal(rt) for rt in road_types])
        type_filter += sql.SQL("]::text[])")

        query = MvtQuery(
            table=RoadLine._meta.db_table,
            attributes=["name", "osm_type"],
            filters=[type_filter],
            layer="transportation",
            transform=False,
            pk="osm_id",
        )
        with connection.cursor() as cursor:
            cursor.execute(query.as_mvt(), asdict(port_moresby))
            tile_response = cursor.fetchone()
            content = tile_response[0]  # type: memoryview
        return bytes(content)

    def test_roads_queryset(self):

        q = Q()
        for road_type, min_zoom in road_osm_types:
            if port_moresby.zoom >= min_zoom:
                q |= Q(osm_type=road_type)

        queryset = RoadLine.objects.filter(q)
        mvtquery = MvtQuery.from_queryset(queryset)
        params = asdict(port_moresby)
        params.update(mvtquery.query_params)

        with connection.cursor() as cursor:
            cursor.execute(mvtquery.as_mvt(), params)
            tile_response = cursor.fetchone()
            content = tile_response[0]  # type: memoryview
        return bytes(content)


class FromQueryTestCase(TestCase):
    def test_from_query(self):

        with connection.cursor() as cursor:
            cursor.execute(MvtQuery.from_model(RoadLine).as_mvt(), asdict(port_moresby))
            tile_response = cursor.fetchone()
            content = tile_response[0]  # type: memoryview
        return bytes(content)

    def test_from_qs(self):

        queryset = RoadLine.objects.filter(pk__in=[1])
        mvtquery = MvtQuery.from_queryset(queryset)
        params = asdict(port_moresby)
        params.update(mvtquery.query_params)

        with connection.cursor() as cursor:
            cursor.execute(mvtquery.as_mvt(), params)
            tile_response = cursor.fetchone()
            content = tile_response[0]  # type: memoryview
        return bytes(content)


class SerializerTestCase(TestCase):
    def setUp(self) -> None:
        BasicPoint.objects.create(name="Pointy point point", geom=Point(1, 1))
        return super().setUp()

    def test_serialize(self):
        url = reverse("test_serializer")
        response = self.client.get(url)
        self.assertEqual(response.status_code, HTTPStatus.OK)

    def test_multi_serialize(self):
        url = reverse("test_multi_serializer")
        response = self.client.get(url)
        self.assertEqual(response.status_code, HTTPStatus.OK)



import re
from dataclasses import dataclass, field
from itertools import count
from typing import Any, Dict, Iterable, List, Optional, Sequence, Tuple, Union

from django.contrib.gis.db.models import GeometryField
from psycopg2 import sql

# To time mvt queries uncomment the following
# from .timer import Timer


@dataclass
class Tile:
    """
    Simple dataclass representing a "tile" instance and additional metadata
    for tile generation: buffer, extent and margin
    http://postgis.net/docs/ST_TileEnvelope.html

    """

    zoom: int
    x: int
    y: int
    buffer: int = 64
    extent: int = 4096

    @staticmethod
    def tile_envelope() -> sql.Composable:
        return sql.SQL("ST_TileEnvelope(%(zoom)s, %(x)s, %(y)s)")

    @staticmethod
    def tile_envelope_margin() -> sql.Composable:
        """
        Returns the ST_TileEnvelope function with a margin
        Note that the margin is only available in postgis 3.1+
        """
        return sql.SQL("ST_TileEnvelope(%(zoom)s, %(x)s, %(y)s, margin => (%(buffer)s / %(extent)s))")


@dataclass
class MvtQuery:
    """
    This is a SQL query generator based on the example
    at https://postgis.net/docs/manual-dev/ST_AsMVT.html
    """

    table: Union[str, sql.Composed, sql.SQL, sql.Identifier]
    # cte's are used to add additional WITH statements. This allows more flexibility in
    # adding Django querysets as MVT layers.
    ctes: Iterable[Tuple[sql.Identifier, Any]] = ()
    # When adding Django querysets with parameters, eg when filtering, there are
    # positional placeholders. These must be converted to named placeholders and included
    # in the `query_params` dict below for the final SQL statement to be correctly
    # handled.
    query_params: Dict[str, Any] = field(default_factory=dict)  # These are args to pass to any internal queryset we use where `table` is a Django queryset
    attributes: List[str] = field(default_factory=list)
    calculated_attributes: Dict[str, Union[sql.Composed, sql.SQL]] = field(default_factory=dict)
    filters: Sequence[sql.Composable] = field(default_factory=list)
    transform: bool = False  # Set to True if source srid is not 3857, but beware performance
    field: str = "geom"
    pk: str = "id"
    layer: str = "default"

    @property
    def json_attributes(self) -> sql.Composed:
        """
        Create a "JSONB_BUILD_OBJECT" clause
        """
        if not self.attributes and not self.calculated_attributes:
            return sql.SQL("").format()
        params = None
        for a in self.attributes:
            if not params:
                params = sql.Literal(a)
            else:
                params += sql.Literal(a)  # Name of the key is the same as the field
            params += sql.Identifier(a)  # The field to use as the value for the JSON

        for field, expression in self.calculated_attributes.items():
            if not params:
                params = sql.Literal(field)
            else:
                params += sql.Literal(field)  # Name of the key is the same as the field
            assert params
            params += expression  # The expression to use as the value for the JSON

        composed_params = params.join(", ")  # type: ignore

        return sql.SQL(", jsonb_build_object({})").format(composed_params)

    @property
    def transformed_geom(self) -> sql.Composable:
        """
        Return an `ST_TRANSFORM` clause
        https://postgis.net/docs/ST_Transform.html
        """
        template = "{field}"
        if self.transform:
            template = f"ST_TRANSFORM({template}, 3857)"
        return sql.SQL(template).format(field=sql.Identifier(self.field))

    @property
    def alias(self) -> sql.Composable:
        """
        The name of the query alias to use for postgresql
        """
        return sql.Identifier(f"mvt_{self.layer}")

    @property
    def where(self) -> sql.Composable:
        return sql.SQL(" AND ") + sql.SQL(" AND ").join(self.filters) if self.filters else sql.SQL("")

    @property
    def as_mvtgeom(self) -> sql.Composable:
        return sql.SQL(
            """
            ST_AsMVTGeom(
                {cg},
                {e},
                extent => %(extent)s,
                buffer => %(buffer)s
            ) AS geom, {pk} {json_attributes}
            FROM {t}
            WHERE {where}
            """
        ).format(
            cg=self.transformed_geom,
            e=Tile.tile_envelope(),
            t=sql.Identifier(self.table) if isinstance(self.table, str) else self.table,
            # Properties of "self"
            pk=sql.Identifier(self.pk),
            json_attributes=self.json_attributes,
            where=self._feature_query,
        )

    @property
    def _feature_query(self) -> sql.Composable:
        """
        Return the sql required for the geometry query
        and other specified filters
        """
        return sql.SQL("""{g} && {m} {where}""").format(
            g=self.transformed_geom,
            m=Tile.tile_envelope(),
            # TODO: Re enable the margin when postgis >= 3.1
            # m=Tile.tile_envelope_margin(),
            where=self.where,
        )

    def get_cte_sql(self) -> sql.Composed:
        """
        The common table expressions we use, composed to
        an appropriate format for the query
        """

        gen = (sql.SQL("{} AS (select {})").format(*_cte) for _cte in (*self.ctes, ((self.alias, self.as_mvtgeom))))
        return (sql.SQL("WITH ") + sql.SQL(",").join(gen)).join(" ")

    def as_mvt(self) -> sql.Composed:
        outer_query = self.get_cte_sql()
        inner_query = sql.SQL(" SELECT ST_AsMVT( {alias}.*, {layer}, %(extent)s, 'geom', {pk}) FROM {alias}" "")
        parameters = dict(alias=self.alias, layer=sql.Literal(self.layer), inner_query=self.as_mvtgeom, pk=sql.Literal(self.pk))
        return (outer_query + inner_query.format(**parameters)).join(" ")

    @classmethod
    def from_model(cls, model, *args, **kwargs) -> "MvtQuery":

        field = kwargs.pop("field", get_geom_field(model))
        attributes = kwargs.pop("attributes", get_model_attributes(model))
        pk = kwargs.get("pk", get_model_pk_field(model))
        transform = model._meta.get_field(field).srid != 3857
        return cls(table=model._meta.db_table, attributes=attributes, field=field, transform=transform, pk=pk, layer=kwargs.pop("layer", model._meta.model_name), **kwargs)

    @classmethod
    def from_queryset(
        cls, queryset, field: str = "geom", attributes: Optional[List[str]] = None, pk: Optional[str] = None, transform: Optional[bool] = False, *args, **kwargs
    ) -> "MvtQuery":
        """
        Takes as input a Django queryset
        This requires more configureation than calling from a model
        as querysets are a little harder to introspect
        """
        # Replace some parts of the Django sql
        # to make it work as a CTE
        django_sql, query_params = convert_to_positional_query(queryset)

        # Our django queryset will become a common table expression AKA "with" statement
        cte_name = sql.Identifier("django_queryset")
        if django_sql.startswith("SELECT"):
            django_sql = django_sql[6:]
        django_sql = django_sql.replace("::bytea", "")

        # When primary key is not specified, detect it from the model
        # Note that if using a queryset this means you need to include it in .values()
        if not pk:
            pk = get_model_pk_field(queryset.model)

        if not attributes:
            attributes = [pk]

        instance = cls(
            table=cte_name,
            ctes=((cte_name, sql.SQL(django_sql)),),
            query_params=query_params,
            attributes=attributes,
            field=field,
            transform=transform,
            pk=pk,
            layer=kwargs.pop("layer", queryset.model._meta.model_name),
            **kwargs,
        )
        return instance


def get_geom_field(model) -> str:
    """
    Returns the first field likely to be a geometry field
    from a model
    """
    fields = model._meta.fields  # type: List
    for field_ in fields:
        if isinstance(field_, GeometryField):
            return field_.db_column or field_.attname
    raise KeyError(f"No geometry field could be identified for {model}")


def get_model_attributes(model) -> List[str]:
    """
    Returns the columns of a model
    """
    fields = model._meta.fields  # type: List
    return [f.db_column or f.attname for f in fields if not isinstance(f, GeometryField)]


def get_model_pk_field(model) -> str:
    fields = model._meta.fields  # type: List
    for pk_field_candidate in fields:
        if pk_field_candidate.primary_key:
            return pk_field_candidate.db_column or pk_field_candidate.attname
    raise KeyError(f"No primary key field could be identified for {model}")


def convert_to_positional_query(queryset):
    """
    From a Django queryset, convert the placeholders to named ones
    in order to enable combining with other parts of this module
    """
    # Initialise a new counter for the named parameters
    c = count()
    django_sql, args = queryset.query.sql_with_params()
    positional_sql = re.sub("%s", lambda x: f"%(param_{next(c)})s", django_sql)
    query_params = {f"param_{i}": param for i, param in enumerate(args)}
    return positional_sql, query_params
